'                  -------------------------------
'					|STATEDESIGN AI FRAMEWORK (SDAI)|
'					|Written by: Caleb Goff         |
'					|Created on: March 22, 2007     |
'					|Last modified: March 22, 2007  |
'                  -------------------------------

'----------------------------------------------------------------
' HOW IT WORKS:
' The base type (see bottom of code file for code framework) is 
' the focus here, and everything else extends from it. Four basic
' functions are available per state. You will need to identify
' multiple base types for multiple AI objects. If you have two
' types of enemies, you'll need two types of base type (e.g.,
' CarState, PlaneState). Certain states will extend from these
' particular base types, and will run your AI. 

' Each state, as previously mentioned, as four functions. All
' functions are required, even if you don't utilize them in your
' game code, as they are hardcoded into the SDAI framework.
' If you don't want them, basic modifications to the source code
' can be made to remove them. The instance function is called
' whenever you set the state, or use the ChangeState function
' in your AI. 

' EnterState is called seperately from ExecuteState, and can be
' used to perform an action upon entering that particular state.
' An example of this would be upon entering a "RunAwayScared"
' state, in which case you might want the AI to first scream and
' wave his arms. The ExitState function works similarily, although
' it is called when you exit a certain state. In the same example
' as used earlier, you might want the AI to sigh and wipe his brow
' before continuing about his business.

' Although a brief introduction, this is the basics of the SDAI
' system. It's an enhanced, and more effecient method of the FSM
' (finite state machine) AI. 
'----------------------------------------------------------------


' This is the state type. Don't mess with this, as it is a-okay!
' We've got three functions, which are called if you are entering,
' exiting, or executing a particular state.

' Different State Types will be required for games with more than
' one AI object type.
Type State Abstract
	
	Function Instance:State() EndFunction
	Function EnterState(obj:AI) EndFunction
	Function ExecuteState(obj:AI) EndFunction
	Function ExitState(obj:AI) EndFunction
	
EndType

' AI Wander State
Type Wander Extends State

	Function Instance:Wander()
		Local instance:Wander = New Wander
		Return instance
	EndFunction
		 
	Function EnterState(obj:AI)
		
		obj.currentStateID = WanderState
		obj.myVehicle.mySprite.GetSteering().WanderOn()
		
	EndFunction
	
	Function ExecuteState(obj:AI)
	
		If Not obj.myVehicle.mySprite.GetSteering().isWanderOn() Then obj.ChangeState(Wander.Instance())
		
	EndFunction
	
	Function ExitState(obj:AI)
		
		obj.myVehicle.mySprite.GetSteering().WanderOff()
		
		
	EndFunction
	
EndType

' AI Null State
Type Nothing Extends State

	Function Instance:Nothing()
		Local instance:Nothing = New Nothing
		Return instance
	EndFunction
		 
	Function EnterState(obj:AI)
		
		'Debug Stuff!
		obj.currentStateID = NullState
		
	EndFunction
	
	Function ExecuteState(obj:AI)

		'Nothing is required, because this state is all about nothing! Woot!
		
	EndFunction
	
	Function ExitState(obj:AI)
		
	EndFunction
	
EndType

Type SeekHealth Extends State
	
	Function Instance:SeekHealth()
		Local instance:SeekHealth = New SeekHealth
		Return instance
	EndFunction
	
	Function EnterState(obj:AI)

		obj.currentStateID = SeekHealthState
		
		Local goalHealth:HealthPickup
		Local tempCount:Int
		
		Local tempVal:Double = 9999
		For Local tHealth:HealthPickUp = EachIn HealthPickUp.List
			If tHealth.Empty = False
				If tempVal > Distance(obj.myVehicle.X,obj.myVehicle.Y,tHealth.X,tHealth.Y)
					tempVal = Distance(obj.myVehicle.X,obj.myVehicle.Y,tHealth.X,tHealth.Y)
					goalHealth = New HealthPickUp
					goalHealth = tHealth
				EndIf
			EndIf
		Next
		
		obj.checkVal = obj.myVehicle.Health
		
		If goalHealth = Null
			obj.ChangeState(Wander.Instance())
		Else
			obj.myVehicle.mySprite.GetSteering().SeekOn(goalHealth.mySprite)
		EndIf
			
	EndFunction
	
	Function ExecuteState(obj:AI)
	
		If obj.myVehicle.Health <> obj.checkVal
			obj.ChangeState(SeekHealth.Instance())
		EndIf
		
	EndFunction
	
	Function ExitState(obj:AI)
	
		obj.myVehicle.mySprite.GetSteering().SeekOff()

	
	EndFunction
	
EndType

Type SeekGun Extends State
	
	Function Instance:SeekGun()
		Local instance:SeekGun = New SeekGun
		Return instance
	EndFunction
	
	Function EnterState(obj:AI)
	
		obj.currentStateID = SeekGunState
		
		Local goalGun:WeaponPickUp = Null
		
		Local tempVal:Double = 9999
		For Local tGun:WeaponPickUp = EachIn WeaponPickUp.List
			If tGun.Empty = False
				If tempVal > Distance(obj.myVehicle.X,obj.myVehicle.Y,tGun.X,tGun.Y)
					tempVal = Distance(obj.myVehicle.X,obj.myVehicle.Y,tGun.X,tGun.Y)
					goalGun = tGun
				EndIf
			EndIf
		Next
		
		obj.checkVal = obj.gDesire
		
		If goalGun = Null
			obj.ChangeState(Wander.Instance())
		Else
			obj.myVehicle.mySprite.GetSteering().SeekOn(goalGun.mySprite)
		EndIf
		
	EndFunction
	
	Function ExecuteState(obj:AI)
	
		If obj.gDesire <> obj.checkVal
			obj.ChangeState(SeekGun.Instance())
		EndIf
		
	EndFunction
	
	Function ExitState(obj:AI)
	
		obj.myVehicle.mySprite.GetSteering().SeekOff()

	
	EndFunction
	
EndType

Type Kill Extends State
	
	Function Instance:Kill()
		Local instance:Kill = New Kill
		Return instance
	EndFunction
	
	Function EnterState(obj:AI)
	
		obj.currentStateID = KillState
		
		Local killTarget:ControlObject
		
		Local tempVal:Double = 99999
		For Local tkillTarget:ControlObject = EachIn ControlObject.List
			If tkillTarget.myVehicle.Dead = False And tkillTarget <> obj
				If tkillTarget.Team <> obj.Team Or obj.Team = 0
					If tempVal > Distance(obj.myVehicle.X,obj.myVehicle.Y,tkillTarget.myVehicle.X,tkillTarget.myVehicle.Y)
						tempVal = Distance(obj.myVehicle.X,obj.myVehicle.Y,tkillTarget.myVehicle.X,tkillTarget.myVehicle.Y)
						killTarget = New ControlObject
						killTarget = tkillTarget
					EndIf
				EndIf
			EndIf
		Next
		
		If killTarget = Null
			obj.ChangeState(Wander.Instance())
		Else
			obj.myVehicle.mySprite.GetSteering().SeekOn(killTarget.myVehicle.mySprite)
		EndIf
		
	EndFunction
	
	Function ExecuteState(obj:AI)
		
	EndFunction
	
	Function ExitState(obj:AI)
	
		obj.myVehicle.mySprite.GetSteering().SeekOff()

	EndFunction
	
EndType 

Rem
	Skeleton Types
	These Skeleton types are provided for your modification needs. They simply give you the framework
	for creating a new state type, or a new state for an object. A little confusing, but you'll get the
	hang of it.

-------------------------------------------------------
Type SkeletonObjectState
	
	Function Instance:State() EndFunction
	Function EnterState(tMiner:Miner) EndFunction
	Function ExecuteState(tMiner:Miner) EndFunction
	Function ExitState(tMiner:Miner) EndFunction
	
EndType
-------------------------------------------------------
Type SkeletonAIState Extends SkeletonObjectState

	Function Instance:SkeletonAIState()
		Local instance:SkeletonAIState = New SkeletonAIState
		Return instance
	EndFunction
		 
	Function EnterState(obj:tObj)
		'Enter State Code Here
	EndFunction
	
	Function ExecuteState(obj:tObj)
		'Execute State Code Here
	EndFunction
	
	Function ExitState(obj:tObj)
		'Exit State Code Here
	EndFunction
	
EndType



